@@ -49,9 +49,9 @@
 #' @param verbose whether to display information (use for debugging purpose). Default is FALSE.
 #' @param overwrite whether to overwrite file or not. Default is FALSE.
 #' Note that if TRUE, it will overwrite exported file if path of 'fileName' and deduced from 'write_to' arguments are different.
-#' Otherwise, you will get an error saying that overwritting source file is not allowed.\cr
+#' Otherwise, you will get an error saying that overwriting source file is not allowed.\cr
 #' Note also that an original file, i.e. generated by IDEAS(R) or INSPIRE(R), will never be overwritten.\cr
-#' Otherwise, you will get an error saying that overwritting original file is not allowed.
+#' Otherwise, you will get an error saying that overwriting original file is not allowed.
 #' @param fullname whether to export daf file with full name of its corresponding cif, if found. Default is TRUE.
 #' If cif can't be found, daf file will be exported with the original cif file name.
 #' @param cifdir the path of the directory to initially look to cif file. Default is dirname(fileName). Only apply when 'fullname' is set to TRUE.
@@ -298,7 +298,7 @@ ExportToDAF <- function(fileName, write_to, pops = list(), regions = list(), fea
         
         if(is_binary) {
           # TODO maybe change endianness reading / writing
-          new_nodes$features = c(new_nodes$features, packBits(intToBits(fid),"raw"), 
+          new_nodes$features = c(new_nodes$features, cpp_uint32_to_raw(fid), 
                                  sapply(feat$val, FUN=function(x) writeBin(object=as.double(x), con=raw(), size = 8, endian = endianness, useBytes = TRUE)))
         } else {
           new_nodes$features = c(new_nodes$features,
@@ -445,7 +445,7 @@ ExportToDAF <- function(fileName, write_to, pops = list(), regions = list(), fea
       }
       cif_name = normalizePath(cif_name, winslash = "/", mustWork = FALSE) # /!\ ask AMNIS using full path produces error while trying to retrieve compensation
     } else {
-      cif_name = cname
+      cif_name = basename(cname)
     } 
     
     # adds pkg version attribute in XML <ASSAY> node /!\ mandatory to prevent overwriting original file
@@ -470,7 +470,7 @@ ExportToDAF <- function(fileName, write_to, pops = list(), regions = list(), fea
     if(!is.null(offset)) {
       seek(towrite, offset)
       # TODO change endianness reading / writing
-      writeBin(object = packBits(intToBits(fid),"raw"), con=towrite, endian = endianness, useBytes = TRUE)
+      writeBin(object = cpp_uint32_to_raw(fid), con=towrite, endian = endianness, useBytes = TRUE)
     }
   }, error = function(e) {
     close(toread)
