@@ -196,7 +196,7 @@ ExportToGallery <- function(...,
   dots = dots[!param_extra] # remove not allowed param
   param_param = names(dots) %in% c("base64_id","base64_att",
                                    "composite","selection","random_seed",
-                                   "removal","add_noise","full_range","force_range")
+                                   "removal","add_noise","full_range","force_range","spatial_correction")
   dots_param = dots[param_param] # keep param_param for objectParam
   dots = dots[!param_param]
   
@@ -229,33 +229,6 @@ ExportToGallery <- function(...,
   title_progress = basename(fileName)
   file_extension = getFileExt(fileName)
   
-  # check objects to extract
-  nobj = as.integer(param$objcount)
-  N = nchar(sprintf("%1.f",abs(nobj-1)))
-  if(missing(objects)) {
-    objects = as.integer(0:(nobj - 1))
-  } else {
-    objects = na.omit(as.integer(objects))
-    tokeep = (objects >= 0) & (objects < nobj)
-    if(length(tokeep) == 0) {
-      if(export == "file") {
-        warning(paste0("ExportToGallery: No objects to export, check the objects you provided.\n",
-                       "Can't create 'write_to' =", write_to, " from file.\n", param$fileName_image),
-                immediate. = TRUE, call. = FALSE)
-        return(invisible(NULL))
-      } else {
-        warning("ExportToGallery: No objects to export, check the objects you provided.\n", immediate. = TRUE, call. = FALSE)
-        return(NULL)
-      }
-    }
-    if(!all(tokeep)) {
-      warning("Some objects that are not in ", fileName, " have been automatically removed from extraction process:\n", paste0(objects[!tokeep], collapse=", "))
-      objects = objects[tokeep]
-    }
-  }
-  extract_max = as.integer(min(extract_max, length(objects)))
-  if(sampling) {objects=sample(objects,extract_max)} else {objects=objects[1:extract_max]}
-  if(length(objects)!=1) if(param$size[2] == 0) stop("'size' width should be provided when 'object' length not equal to one")
   
   # check input offsets if any
   compute_offsets = TRUE
@@ -274,6 +247,24 @@ ExportToGallery <- function(...,
     offsets = suppressMessages(getOffsets(fileName = param$fileName_image, fast = fast, display_progress = display_progress, verbose = verbose))
   }
   
+  # check objects to extract
+  nobj = as.integer(attr(x = offsets, which = "obj_count"))
+  
+  N = nchar(sprintf("%1.f",abs(nobj-1)))
+  if(missing(objects)) {
+    objects = as.integer(0:(nobj - 1))
+  } else {
+    objects = na.omit(as.integer(objects))
+    tokeep = (objects >= 0) & (objects < nobj)
+    if(!all(tokeep)) {
+      warning("Some objects that are not in ", fileName, " have been automatically removed from extraction process:\n", paste0(objects[!tokeep], collapse=", "))
+      objects = objects[tokeep]
+    }
+  }
+  extract_max = as.integer(min(extract_max, length(objects)))
+  if(sampling) {objects=sample(objects,extract_max)} else {objects=objects[1:extract_max]}
+  if(length(objects)!=1) if(param$size[2] == 0) stop("'size' width should be provided when 'object' length not equal to one")
+  
   # check export/write_to
   overwritten = FALSE
   if(export != "matrix") {
@@ -317,21 +308,32 @@ ExportToGallery <- function(...,
   }
   
   # extract objects
-  sel = split(objects, ceiling(seq_along(objects)/20))
-  L = length(sel)
+  sel = subsetOffsets(offsets = offsets, objects = objects, image_type = image_type)
+  sel = split(sel, ceiling(seq_along(sel)/20))
+  L=length(sel)
+  if(L == 0) {
+    if(export == "file") {
+      warning(paste0("ExportToGallery: No objects to export, check the objects you provided.\n",
+                     "Can't create 'write_to' =", write_to, " from file.\n", param$fileName_image),
+              immediate. = TRUE, call. = FALSE)
+      return(invisible(NULL))
+    } else {
+      warning("ExportToGallery: No objects to export, check the objects you provided.\n", immediate. = TRUE, call. = FALSE)
+      return(NULL)
+    }
+  }
   tryCatch({
     tryCatch({
       if(display_progress) {
         pb = newPB(session = dots$session, min = 0, max = L, initial = 0, style = 3)
         ans = lapply(1:L, FUN = function(i) {
           setPB(pb, value = i, title = title_progress, label = "exporting objects")
-          do.call(what = "objectExtract", args = c(list(ifd = getIFD(fileName = param$fileName_image,
-                                                                     offsets = subsetOffsets(offsets = offsets, objects = sel[[i]], image_type = image_type),
-                                                                     trunc_bytes = 8, 
-                                                                     force_trunc = FALSE, 
-                                                                     verbose = verbose, 
-                                                                     verbosity = verbosity,
-                                                                     bypass = TRUE), 
+          do.call(what = "objectExtract", args = c(list(ifd = lapply(sel[[i]],
+                                                                     FUN = function(off) cpp_getTAGS(fname = param$fileName_image,
+                                                                                                     offset = off,
+                                                                                                     trunc_bytes = 1, 
+                                                                                                     force_trunc = TRUE, 
+                                                                                                     verbose = verbose)),
                                                         param = param,
                                                         verbose = verbose,
                                                         bypass = TRUE),
@@ -339,13 +341,12 @@ ExportToGallery <- function(...,
         })
       } else {
         ans = lapply(1:L, FUN = function(i) {
-          do.call(what = "objectExtract", args = c(list(ifd = getIFD(fileName = param$fileName_image,
-                                                                     offsets = subsetOffsets(offsets = offsets, objects = sel[[i]], image_type = image_type),
-                                                                     trunc_bytes = 8, 
-                                                                     force_trunc = FALSE, 
-                                                                     verbose = verbose, 
-                                                                     verbosity = verbosity,
-                                                                     bypass = TRUE), 
+          do.call(what = "objectExtract", args = c(list(ifd = lapply(sel[[i]],
+                                                                     FUN = function(off) cpp_getTAGS(fname = param$fileName_image,
+                                                                                                     offset = off,
+                                                                                                     trunc_bytes = 1, 
+                                                                                                     force_trunc = TRUE, 
+                                                                                                     verbose = verbose)),
                                                         param = param,
                                                         verbose = verbose,
                                                         bypass = TRUE),
@@ -392,7 +393,7 @@ ExportToGallery <- function(...,
       id = ids[i]
       d = dim(img[[1]])
       if(objects[i] != id) warning(paste0("Extracted object_ids:", id, " differs from expected one:", objects[i]), call. = FALSE, immediate. = TRUE)
-      if(add_ids > 0) img[[add_ids]] = addText(img[[add_ids]], text = id, col, xoff = 1, yoff = 1)
+      if(add_ids > 0) img[[add_ids]] = addText(img[[add_ids]], text = num_to_string(id), col, xoff = 1, yoff = 1)
       array(do.call(rbind, args = lapply(img, FUN = function(i) { apply(i, 3, rbind)})), dim = c(d[1], d[2] * length(img), 3))
     })
     d = dim(ans[[1]])
