@@ -147,23 +147,6 @@ ExtractImages_toFile <- function(...,
   fileName = param$fileName_image
   title_progress = basename(fileName)
   
-  # check objects to extract
-  nobj = as.numeric(param$objcount)
-  if(missing(objects)) {
-    objects = as.integer(0:(nobj - 1))
-  } else {
-    objects = na.omit(as.integer(objects))
-    tokeep = (objects >= 0) & (objects < nobj)
-    if(length(tokeep) == 0) {
-      warning("ExtractImages_toFile: No objects to extract, check the objects you provided.", immediate. = TRUE, call. = FALSE)
-      return(invisible(NULL))
-    }
-    if(!all(tokeep)) {
-      warning("Some objects that are not in ", fileName, " have been automatically removed from extraction process:\n", paste0(objects[!tokeep], collapse=", "))
-      objects = objects[tokeep]
-    }
-  }
-  
   # check input offsets if any
   compute_offsets = TRUE
   if(length(offsets) != 0) {
@@ -181,21 +164,39 @@ ExtractImages_toFile <- function(...,
     offsets = suppressMessages(getOffsets(fileName = param$fileName_image, fast = fast, display_progress = display_progress, verbose = verbose))
   }
   
+  # check objects to extract
+  nobj = as.integer(attr(x = offsets, which = "obj_count"))
+  
+  if(missing(objects)) {
+    objects = as.integer(0:(nobj - 1))
+  } else {
+    objects = na.omit(as.integer(objects))
+    tokeep = (objects >= 0) & (objects < nobj)
+    if(!all(tokeep)) {
+      warning("Some objects that are not in ", fileName, " have been automatically removed from extraction process:\n", paste0(objects[!tokeep], collapse=", "))
+      objects = objects[tokeep]
+    }
+  }
+  
   # extract objects
-  sel = split(objects, ceiling(seq_along(objects)/20))
+  sel = subsetOffsets(offsets = offsets, objects = objects, image_type = "img")
+  sel = split(sel, ceiling(seq_along(sel)/20))
   L=length(sel)
+  if(L == 0) {
+    warning("ExtractImages_toFile: No objects to extract, check the objects you provided.", immediate. = TRUE, call. = FALSE)
+    return(invisible(NULL))
+  }
   if(display_progress) {
     pb = newPB(session = dots$session, min = 0, max = L, initial = 0, style = 3)
     on.exit(endPB(pb))
     ans = lapply(1:L, FUN=function(i) {
       setPB(pb, value = i, title = title_progress, label = "exporting images to file")
-      do.call(what = "objectExtract", args = c(list(ifd = getIFD(fileName = param$fileName_image,
-                                                                 offsets = subsetOffsets(offsets = offsets, objects = sel[[i]], image_type = "img"),
-                                                                 trunc_bytes = 8, 
-                                                                 force_trunc = FALSE, 
-                                                                 verbose = verbose, 
-                                                                 verbosity = verbosity,
-                                                                 bypass = TRUE), 
+      do.call(what = "objectExtract", args = c(list(ifd = lapply(sel[[i]],
+                                                                 FUN = function(off) cpp_getTAGS(fname = param$fileName_image,
+                                                                                                 offset = off,
+                                                                                                 trunc_bytes = 1, 
+                                                                                                 force_trunc = TRUE, 
+                                                                                                 verbose = verbose)),
                                                     param = param,
                                                     verbose = verbose,
                                                     bypass = TRUE),
@@ -203,13 +204,12 @@ ExtractImages_toFile <- function(...,
     })
   } else{
     ans = lapply(1:L, FUN=function(i) {
-      do.call(what = "objectExtract", args = c(list(ifd = getIFD(fileName = param$fileName_image,
-                                                                 offsets = subsetOffsets(offsets = offsets, objects = sel[[i]], image_type = "img"),
-                                                                 trunc_bytes = 8, 
-                                                                 force_trunc = FALSE, 
-                                                                 verbose = verbose, 
-                                                                 verbosity = verbosity,
-                                                                 bypass = TRUE), 
+      do.call(what = "objectExtract", args = c(list(ifd = lapply(sel[[i]],
+                                                                 FUN = function(off) cpp_getTAGS(fname = param$fileName_image,
+                                                                                                 offset = off,
+                                                                                                 trunc_bytes = 1, 
+                                                                                                 force_trunc = TRUE, 
+                                                                                                 verbose = verbose)),
                                                     param = param,
                                                     verbose = verbose,
                                                     bypass = TRUE),
