@@ -53,7 +53,6 @@
 #'                   'to install extra files required to run this example.'))
 #' }
 #' @return an integer vector of class `IFC_offset` of IFDs offsets found in XIF file.
-#' If no offsets is found an error is thrown.
 #' @export
 getOffsets <- function(fileName, fast = TRUE, display_progress = TRUE, verbose = FALSE) {
   if(missing(fileName)) stop("'fileName' can't be missing")
@@ -67,17 +66,28 @@ getOffsets <- function(fileName, fast = TRUE, display_progress = TRUE, verbose =
   display_progress = as.logical(display_progress); assert(display_progress, len = 1, alw = c(TRUE, FALSE))
   verbose = as.logical(verbose); assert(verbose, len = 1, alw = c(TRUE, FALSE))
   fileName = normalizePath(fileName, winslash = "/", mustWork = FALSE)
-  obj_count = as.integer(getInfo(fileName, warn = FALSE, force_default = TRUE, display_progress = FALSE)$objcount)
-  if(fast) {
-    offsets = cpp_getoffsets_noid(fileName, obj_count = obj_count, display_progress = display_progress, verbose = verbose)
+  
+  XIF_test = testXIF(fileName)
+  obj_count = attr(XIF_test, "obj_count")
+  obj_estimated = attr(XIF_test, "obj_estimated")
+  if((obj_count != obj_estimated) && display_progress) message("offsets extraction is based on an estimation of total number of objects within the file.\nConsequently, it is normal behaviour if progress bar can not reach or exceed 100%.")
+  # obj_count = as.integer(getInfo(fileName, warn = FALSE, force_default = TRUE, display_progress = FALSE)$objcount)
+  
+  type = as.integer(XIF_test == 1) + 1L
+  if(fast || (XIF_test < 0)) {
+    offsets = cpp_getoffsets_noid(fileName, obj_count = obj_estimated, display_progress = display_progress, verbose = verbose)
+    offsets = offsets[offsets != 0]
     offsets_1 = offsets[1]
     offsets = offsets[-1]
-    if(obj_count > 0) if(length(offsets) != obj_count * 2) stop("Number of offsets found is different from expected object count.")
+    if(obj_count > 0) if(length(offsets) != obj_estimated * type) stop("Number of offsets found is different from expected object count.")
+    
     message("Offsets were extracted from XIF file with fast method.\nCorrect mapping between offsets and objects ids is not guaranteed.")
   } else {
-    offsets = as.data.frame(do.call(what = "cbind", args = cpp_getoffsets_wid(fileName, obj_count = obj_count, display_progress = display_progress, verbose = verbose)), stringsAsFactors = FALSE)
+    offsets = as.data.frame(do.call(what = "cbind", args = cpp_getoffsets_wid(fileName, obj_count = obj_estimated / (as.integer(XIF_test != 1) + 1L), display_progress = display_progress, verbose = verbose)), stringsAsFactors = FALSE)
+    offsets = offsets[offsets$OFFSET != 0, ]
     offsets_i = offsets[offsets$TYPE == 2, ]
     offsets_m = offsets[offsets$TYPE == 3, ]
+    if(type == 2) if(length(offsets_i$OBJECT_ID) != length(offsets_m$OBJECT_ID)) stop("Offsets contain different numbers of 'img' and 'msk'.")
     
     ORD_i = order(offsets_i$OBJECT_ID)
     offsets_img = offsets_i$OFFSET[ORD_i]
@@ -87,20 +97,26 @@ getOffsets <- function(fileName, fast = TRUE, display_progress = TRUE, verbose =
     offsets_1 = offsets[offsets$TYPE == 1, ]
     offsets_1 = offsets_1$OFFSET
     
-    if(length(offsets_img) != length(offsets_msk)) stop("Offsets contain different numbers of 'img' and 'msk'.")
     if(obj_count > 0) if(length(offsets_img) != obj_count) stop("Number of offsets found is different from expected object count.")
     offsets = as.integer(apply(cbind(offsets_img, offsets_msk), 1, FUN=function(i) i))
   }
-  if(length(offsets) != 0) {
-    if(obj_count <= 0) obj_count = length(offsets) / 2
+  if(length(offsets) == 0) {
+    obj_count = 0
+  } else {
+    if(obj_count <= 0) obj_count = length(offsets) / type
     N = nchar(sprintf("%1.f",abs(obj_count-1)))
-    names(offsets) = c(paste0(c("img_", "msk_"), rep(sprintf(paste0("%0",N,".f"), 0:(obj_count-1)), each = 2)))
-    attr(offsets, "all") = offsets
-    attr(offsets, "fileName_image") = fileName
-    attr(offsets, "checksum") = checksumXIF(fileName)
-    attr(offsets, "class") = c("IFC_offset")
-    attr(offsets, "first") = offsets_1 
-    return(offsets)
+    if(type == 1) {
+      names(offsets) = paste0("img_", rep(sprintf(paste0("%0",N,".f"), 0:(obj_count-1))))
+    } else {
+      names(offsets) = paste0(c("img_", "msk_"), rep(sprintf(paste0("%0",N,".f"), 0:(obj_count-1)), each = 2))
+    }
   }
-  stop(paste0("No IFD offsets found in\n", fileName))
+  attr(offsets, "all") = offsets
+  attr(offsets, "fileName_image") = fileName
+  attr(offsets, "checksum") = checksumXIF(fileName)
+  attr(offsets, "obj_count") = obj_count
+  attr(offsets, "first") = offsets_1 
+  attr(offsets, "test") = XIF_test
+  attr(offsets, "class") = c("IFC_offset")
+  return(offsets)
 }
