@@ -27,48 +27,26 @@
 # along with IFC. If not, see <http://www.gnu.org/licenses/>.                  #
 ################################################################################
 
-#' @title LinLog Transformation of Ticks and Labels
-#' @description Helper to rescale and label axes when linlog transformation is used
-#' @keywords internal
-scale_trans=function(hyper, b=10, lin_comp=log(b)){
-  at=outer(-10:10,-10:10,FUN=function(m,p) {m*b^p})
-  toKeep=abs(at)>=hyper
-  at=c(at[toKeep],seq(-hyper,hyper,length.out = 9)[2:8])
-  at=smoothLinLog(x=at, hyper=hyper, base=b, lin_comp=lin_comp)
-  lab=outer(-10:10,-10:10,FUN=function(m,p) {paste0(m,"*",b,"^",p)})
-  lab[grep(paste0(1,"\\*",b,"\\^"),lab,invert=TRUE)]=""
-  lab=c(lab[toKeep],seq(-hyper,hyper,length.out = 9)[2:8])
-  lab=gsub("\\*10\\^","e",lab)
-  return(list(at=at,labels=lab))
-}
 
-#' @title LinLog Transformation for IFC Graphs Plotting Scales
-#' @description Helper to rescale and label axes when linlog transformation is used
+#' @title Scale Constructor for IFC Graphs Plotting
+#' @description Helper to rescale and label axes
 #' @keywords internal
 myScales=function(x=list(), y=list()) {
   if(length(x$alternating)==0) x$alternating=1
   if(length(x$tck)==0) x$tck=c(TRUE,FALSE)
   if(length(x$hyper)==0) x$hyper="P"
-  if(length(x$b)==0) x$b=10
-  if(length(x$lin_comp)==0) x$lin_comp=log(x$b)
   if(length(x$rot)==0) x$rot=45
   
   if(length(y$alternating)==0) y$alternating=1
   if(length(y$tck)==0) y$tck=c(TRUE,FALSE)
   if(length(y$hyper)==0) y$hyper="P"
-  if(length(y$b)==0) y$b=10
-  if(length(y$lin_comp)==0) y$lin_comp=log(y$b)
   if(length(y$rot)==0) y$rot=0
   
   x_scale=list("alternating"=x$alternating,"tck"=x$tck,"rot"=x$rot)
   y_scale=list("alternating"=y$alternating,"tck"=y$tck,"rot"=y$rot)
-  if(x$hyper!="P") x_scale=c(x_scale, do.call("scale_trans", list(hyper=x$hyper,b=x$b,lin_comp=x$lin_comp)))
-  if(y$hyper!="P") y_scale=c(y_scale, do.call("scale_trans", list(hyper=y$hyper, b=y$b, lin_comp=y$lin_comp)))
+  x_scale=c(x_scale, base_axis_constr(x$lim, x$hyper))
+  y_scale=c(y_scale, base_axis_constr(y$lim, y$hyper))
   
-  if(length(x$at)!=0) x_scale=c(x_scale,"at"=x$at)
-  if(length(x$lab)!=0) x_scale=c(x_scale,"labels"=x$labels)
-  if(length(y$at)!=0) y_scale=c(y_scale,"at"=y$at)
-  if(length(y$lab)!=0) y_scale=c(y_scale,"labels"=y$labels)
   return(list("x"=x_scale,"y"=y_scale))
 }
 
@@ -82,7 +60,8 @@ calcDensity <- getFromNamespace(x = ".smoothScatterCalcDensity", ns = "grDevices
 #' @description Helper to map density to colors
 #' @source derived from \pkg{grDevices} R Core Team, Florian Hahne at FHCRC, originally
 #' @keywords internal
-densCols=function (x, y = NULL, nbin = 128, bandwidth, colramp = colorRampPalette(c("blue","green","red")), transformation=function(x) (x^0.125)) {
+densCols=function (x, y = NULL, nbin = 128, bandwidth, colramp = colorRampPalette(c("blue","green","red")), transformation=function(x) (asinh(x))) {
+  x_features = attr(x, "features")
   xy <- xy.coords(x, y)
   select <- is.finite(xy$x) & is.finite(xy$y)
   x <- cbind(xy$x, xy$y)[select, ]
@@ -91,7 +70,17 @@ densCols=function (x, y = NULL, nbin = 128, bandwidth, colramp = colorRampPalett
   xbin <- cut(x[, 1], mkBreaks(map$x1), labels = FALSE)
   ybin <- cut(x[, 2], mkBreaks(map$x2), labels = FALSE)
   dens <- map$fhat[cbind(xbin, ybin)]
-  dens <- transformation(dens) # slightly modifyed: a transformation function has been introduced 
+  check_fun <- try(suppressWarnings(formals(transformation)), silent = TRUE)
+  if(!inherits(check_fun, what="try-error")) {
+    if("features" %in% names(check_fun)) {
+      args = list(features = x_features)
+    } else {
+      args = list(x = dens)
+    }
+    dens <- do.call(what = transformation, args = args)  
+  } else {
+    dens = x_features
+  }
   dens[is.na(dens)] <- 0
   colpal <- cut(dens, length(dens), labels = FALSE)
   cols <- rep(NA_character_, length(select))
@@ -112,6 +101,15 @@ colConv=function(col){
   }))
 }
 
+#' @title Hexadecimal to Integer Color Conversion
+#' @description Helper to convert color from hex
+#' @keywords internal
+inv_colConv=function(col){
+  x = col2rgb(col,T)
+  paste0(apply(x, 2, FUN = function(i) { sum(i * c(2^16,2^8,2^1,0)) - 2^24 - i[3] }),"|",collapse="")
+}
+
+
 #' @title Histogram Constructor
 #' @name hist_constr
 #' @description Helper to construct histogram 
@@ -210,108 +208,134 @@ toEllipse=function(gate, theta=2*pi, npoints=100) {
   return(list("x"=xx*cos(2*pi-theta)+yy*sin(2*pi-theta)+ell[3],"y"=yy*cos(2*pi-theta)-xx*sin(2*pi-theta)+ell[4]))
 }
 
-################################################################################
-#                       functions to use base plot                             #
-################################################################################
-#' @title Axis Constructor for 'base' Plot
+#' @title Axis Constructor
 #' @name base_axis_constr
-#' @description Helper to rescale and label axes when linlog transformation is used.
+#' @description Helper to rescale and label axes when linlog / asinh transformation is used.
 #' @param lim vector of length 2 of axis extents.
-#' @param hyper value where transition between Lin/Log is applied. Defaut is "P".
+#' @param trans transformation applied. Defaut is "P".
 #' @param nint positive integer value indicating (approximately) the desired number of intervals. Default is 10.
 #' @keywords internal
-base_axis_constr = function(lim, hyper = "P", nint = 10) {
+base_axis_constr = function(lim, trans = "P", nint = 10) {
   nint = na.omit(as.integer(nint)); assert(nint, len = 1, typ = "integer")
-  assert(hyper, len = 1)
-  if(hyper == "P") {
+  assert(trans, len = 1)
+  trans_ = parseTrans(trans)
+  if(trans_$what %in% c("smoothLinLog","smoothAsinh")) {
+    hyper = formals(trans_$what)$hyper
+    if(length(trans_$args$hyper)!=0) hyper = trans_$args$hyper
+    base = formals(trans_$what)$base
+    if(length(base) == 0) base = 10
+    if(length(trans_$args$base)!=0) base = trans_$args$base
+    n_ticks = 0
+    p_ticks = 0
+    neg_log_ticks = 0
+    pos_log_ticks = 0
+    # compute range without expansion in original scale
+    ran = diff(lim) / 1.14 * c(0.07, -0.07) + lim
+    ran = applyTrans(ran, trans_, inverse = TRUE)
+    n_ticks = max(ran[1], -hyper)
+    p_ticks = min(ran[2], hyper)
+    
+    # identify pos and neg base pow within displayed range
+    ran_ = ran + c(hyper, -hyper)
+    ran_ = sign(ran_) * log(abs(ran_) / hyper, base)
+    if(ran_[1] < 0) neg_log_ticks = floor(ran_[1])
+    if(ran_[2] > 0) pos_log_ticks = ceiling(ran_[2])
+    tot = pos_log_ticks - neg_log_ticks + 
+      ifelse((neg_log_ticks < 0) && (n_ticks >= -hyper), 0.5, 0) + 
+      ifelse((pos_log_ticks > 0) && (p_ticks <= hyper), 0.5, 0)
+    # create ticks and labels
+    ticks_at = c()
+    ticks_lab = c()
+    if(neg_log_ticks != 0) {
+      at = round(sort(unique(c(outer(1:base, (max(abs(neg_log_ticks),abs(pos_log_ticks))-1):0 + log(hyper, base),
+                               FUN=function(m,p) {-m*base^p}))), decreasing = FALSE),10)
+      at_scaled = applyTrans(at, trans_)
+      at_lab = rep("", length(at_scaled))
+      neg_nint = as.integer(-neg_log_ticks / tot * nint)
+      if(neg_nint > 0) {
+        if(length(at) < (1 * nint)) {
+          at_lab = formatC(x = at, format = "g", width = -1, digits = 1, drop0trailing = TRUE) 
+        } else {
+          if(base == 10) {
+            pretty_lab = round(-axisTicks(log10(-range(at)), log = TRUE, nint = neg_nint),10)
+          } else {
+            pretty_lab = round(-base^axisTicks(log(-range(at), base), log = FALSE, nint = neg_nint),10)
+          }
+          # log = log10(-at[at %in% pretty_lab])
+          # tmp = (log - floor(log)) == 0
+          # log[!tmp] <- floor(log[!tmp])
+          # mul = at[at %in% pretty_lab] / 10^log
+          # at_lab[at %in% pretty_lab] <- parse(text=sprintf("%i%%.%%10^~~%i", mul, log))
+          at_lab[at %in% pretty_lab] = formatC(x = at[at %in% pretty_lab] , format = "g", width = -1, digits = 2, drop0trailing = TRUE) 
+        }
+      }
+      ticks_at = c(ticks_at, at_scaled)
+      ticks_lab = c(ticks_lab, at_lab)
+    }
+    if(n_ticks >= -hyper) {
+      at = axisTicks(c(n_ticks,0), log = FALSE, nint = ceiling(3*nint*applyTrans(abs(n_ticks), trans_)/diff(lim)/tot))
+      at = at[at > n_ticks]
+      if(length(at) > 0) {
+        at_scaled = applyTrans(at, trans_)
+        at_lab = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE) 
+        ticks_at = c(ticks_at, at_scaled)
+        ticks_lab = c(ticks_lab, at_lab)
+      }
+    }
+    if(p_ticks <= hyper) {
+      at = axisTicks(c(0,p_ticks), log = FALSE, nint = ceiling(3*nint*applyTrans(abs(p_ticks), trans_)/diff(lim)/tot))
+      at = at[at < p_ticks]
+      if(length(at) > 0) {
+        at_scaled = applyTrans(at, trans_)
+        at_lab = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE) 
+        ticks_at = c(ticks_at, at_scaled)
+        ticks_lab = c(ticks_lab, at_lab)
+      }
+    }
+    if(pos_log_ticks != 0) {
+      at = round(sort(unique(c(outer(1:base, 0:(max(abs(neg_log_ticks),abs(pos_log_ticks))-1) + log(hyper, base),
+                               FUN=function(m,p) {m*base^p}))), decreasing = FALSE),10)
+      at_scaled = applyTrans(at, trans_)
+      at_lab = rep("", length(at_scaled))
+      pos_nint = as.integer(pos_log_ticks / tot * nint)
+      if(pos_nint > 0) {
+        if(length(at) < (1 * nint)) {
+          at_lab = formatC(x = at, format = "g", width = -1, digits = 1, drop0trailing = TRUE) 
+        } else {
+          if(base == 10) {
+            pretty_lab = round(axisTicks(log10(range(at)), log = TRUE, nint = pos_nint),10)
+          } else {
+            pretty_lab = round(base^axisTicks(log(range(at), base), log = FALSE, nint = pos_nint),10)
+          }
+          at_lab[at %in% pretty_lab] = formatC(x = at[at %in% pretty_lab] , format = "g", width = -1, digits = 2, drop0trailing = TRUE) 
+        }
+      }
+      ticks_at = c(ticks_at, at_scaled)
+      ticks_lab = c(ticks_lab, at_lab)
+    }
+    keep = (ticks_at >= lim[1]) & (ticks_at <= lim[2])
+    dup = duplicated(ticks_at)
+    ticks_at = ticks_at[!dup & keep]
+    ticks_lab = ticks_lab[!dup & keep]
+    if(length(ticks_at) < 2) {
+      at = signif(seq(from = lim[1], to = lim[2], length.out = nint), 3)
+      ticks_at = applyTrans(at, trans_)
+      ticks_lab = at
+    }
+    ticks_lab = ticks_lab[is.finite(ticks_at)]
+    ticks_at = ticks_at[is.finite(ticks_at)]
+    # trick to fix 
+    # grid.Call(C_textBounds, as.graphicsAnnot(x$label), x$x, x$y,  : 
+    # polygon edge not found (zero-width or zero-height?)
+    # Calls: <Anonymous> -> heightDetails -> heightDetails.text -> grid.Call
+    ticks_lab[ticks_lab == ""] <- " "
+    ord = order(ticks_at)
+    return(list("at" = ticks_at[ord], "labels" = ticks_lab[ord]))
+  } else {
     at = axisTicks(lim, log = FALSE, nint = nint)
-    return(list("at" = at, "label" = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE)))
+    at = at[is.finite(at)]
+    return(list("at" = at, "labels" = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE)))
   }
-  hyper = na.omit(as.integer(hyper)); assert(hyper, len = 1, typ = "integer")
-  n_ticks = 0
-  p_ticks = 0
-  neg_log_ticks = 0
-  pos_log_ticks = 0
-  ran = lim / c(1.07)
-  ran = inv_smoothLinLog(ran, hyper)
-  n_ticks = max(ran[1], -hyper)
-  p_ticks = min(ran[2], hyper)
-  ran = ran + c(hyper, -hyper)
-  ran = sign(ran) * log10(abs(ran) / hyper)
-  dif = diff(ran)
-  if(ran[1] < 0) neg_log_ticks = floor(ran[1])
-  if(ran[2] > 0) pos_log_ticks = ceiling(ran[2])
-  tot = pos_log_ticks - neg_log_ticks + 
-    ifelse((neg_log_ticks < 0) && (n_ticks <= -hyper), 0.5, 0) + 
-    ifelse((pos_log_ticks > 0) && (p_ticks >= hyper), 0.5, 0)
-  
-  ticks_at = c()
-  ticks_lab = c()
-  if(neg_log_ticks != 0) {
-    at = sort(unique(c(outer(1:10, (abs(neg_log_ticks)-1):0 + log10(hyper),
-                             FUN=function(m,p) {-m*10^p}))), decreasing = FALSE)
-    at_scaled = smoothLinLog(at, hyper = hyper)
-    at_lab = rep("", length(at_scaled))
-    neg_nint = as.integer(-neg_log_ticks / tot * 3 * nint)
-    if(neg_nint > 0) {
-      if(length(at) < (1 * nint)) {
-        at_lab = formatC(x = at, format = "g", width = -1, digits = 1, drop0trailing = TRUE) 
-      } else {
-        pretty_lab = -axisTicks(log10(-range(at)), log = TRUE, nint = neg_nint)
-        at_lab[at %in% pretty_lab] <- formatC(x = at[at %in% pretty_lab], format = "g", width = -1, digits = 1)
-      }
-    }
-    ticks_at = c(ticks_at, at_scaled)
-    ticks_lab = c(ticks_lab, at_lab)
-  }
-  if(n_ticks <= -hyper) {
-    at = axisTicks(c(-hyper,0), log = FALSE, nint = ceiling(1/tot*nint))
-    at = at[at > -hyper]
-    if(length(at) > 0) {
-      at_scaled = smoothLinLog(at, hyper = hyper)
-      at_lab = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE) 
-      ticks_at = c(ticks_at, at_scaled)
-      ticks_lab = c(ticks_lab, at_lab)
-    }
-  }
-  if(p_ticks >= hyper) {
-    at = axisTicks(c(0,hyper), log = FALSE, nint = ceiling(1/tot*nint))
-    at = at[at < hyper]
-    if(length(at) > 0) {
-      at_scaled = smoothLinLog(at, hyper = hyper)
-      at_lab = formatC(x = at, format = "g", width = -1, digits = 4, drop0trailing = TRUE) 
-      ticks_at = c(ticks_at, at_scaled)
-      ticks_lab = c(ticks_lab, at_lab)
-    }
-  }
-  if(pos_log_ticks != 0) {
-    at = sort(unique(c(outer(1:10, 0:(abs(pos_log_ticks)-1) + log10(hyper),
-                             FUN=function(m,p) {m*10^p}))), decreasing = FALSE)
-    at_scaled = smoothLinLog(at, hyper = hyper)
-    at_lab = rep("", length(at_scaled))
-    pos_nint = as.integer(pos_log_ticks / tot * 3 * nint)
-    if(pos_nint > 0) {
-      if(length(at) < (1 * nint)) {
-        at_lab = formatC(x = at, format = "g", width = -1, digits = 1, drop0trailing = TRUE) 
-      } else {
-        pretty_lab = axisTicks(log10(range(at)), log = TRUE, nint = pos_nint)
-        at_lab[at %in% pretty_lab] <- formatC(x = at[at %in% pretty_lab], format = "g", width = -1, digits = 1)
-      }
-    }
-    ticks_at = c(ticks_at, at_scaled)
-    ticks_lab = c(ticks_lab, at_lab)
-  }
-  keep = (ticks_at >= lim[1]) & (ticks_at <= lim[2])
-  dup = duplicated(ticks_at)
-  ticks_at = ticks_at[!dup & keep]
-  ticks_lab = ticks_lab[!dup & keep]
-  if(length(ticks_at) < 2) {
-    at = signif(seq(from = lim[1], to = lim[2], length.out = nint), 3)
-    ticks_at = smoothLinLog(at, hyper = hyper)
-    ticks_lab = at
-  }
-  ord = order(ticks_at)
-  return(list("at" = ticks_at[ord], "label" = ticks_lab[ord]))
 }
 
 #' @title Histogram Constructor for 'base' Plot
@@ -391,6 +415,9 @@ convert_to_baseplot = function(obj) {
          main = trunc_string(obj$input$title, obj$input$trunc_labels), 
          xlab = trunc_string(obj$input$xlab, obj$input$trunc_labels),
          ylab = trunc_string(obj$input$ylab, obj$input$trunc_labels),
+         cex.lab = obj$plot$par.settings$par.xlab.text$cex,
+         cex.main = obj$plot$par.settings$par.main.text$cex,
+         cex.axis = obj$plot$par.settings$axis.text$cex,
          col = "transparent", border = "transparent", freq = obj$input$type == "count",
          breaks = br, axes = FALSE)
     if(length(displayed) > 0) {
@@ -412,50 +439,65 @@ convert_to_baseplot = function(obj) {
     if(obj$input$type == "density") {
       pch=16
       col = "white"
-      if(nrow(obj$input$data) > 0)
-        col=densCols(x = obj$input$data$x2, y = obj$input$data$y2,
+      if((nrow(obj$input$data) > 0) && any(obj$input$subset)) if(nrow(obj$input$data[obj$input$subset,]) > 0) 
+        col=densCols(x = structure(obj$input$data$x2[obj$input$subset], features=attr(obj$input$data,"features")),
+                     y = obj$input$data$y2[obj$input$subset],
                      colramp=colorRampPalette(colConv(basepop[[1]][c("densitycolorsdarkmode","densitycolorslightmode")][[obj$input$mode]])),
                      nbin=obj$input$bin,
                      transformation=obj$input$trans)
-      
-      plot(x = obj$input$data$x2, y = obj$input$data$y2,
-           xlim = Xlim , ylim = Ylim ,
+      plot(x = obj$input$data$x2[obj$input$subset],
+           y = obj$input$data$y2[obj$input$subset],
+           xlim = Xlim , ylim = Ylim,
            main = obj$input$title,
            xlab = trunc_string(obj$input$xlab, obj$input$trunc_labels),
            ylab = trunc_string(obj$input$ylab, obj$input$trunc_labels),
+           cex.lab = obj$plot$par.settings$par.xlab.text$cex,
+           cex.main = obj$plot$par.settings$par.main.text$cex,
+           cex.axis = obj$plot$par.settings$axis.text$cex,
            pch = pch, col = col, 
            axes = FALSE)
+      if(!(inherits(obj$input$trans, what="function") || 
+           !inherits(try(suppressWarnings(formals(obj$input$trans)), silent = TRUE), what="try-error"))) mtext(side = 3, line = 0.25, adj = 0.5, obj$input$trans)
     } else {
       if(obj$input$precision == "full") {
         disp = disp_n[length(displayed)]
-        plot(x = obj$input$data[obj$input$data[,disp], "x2"], y = obj$input$data[obj$input$data[,disp], "y2"],
+        plot(x = obj$input$data[obj$input$data[,disp], "x2"][obj$input$subset],
+             y = obj$input$data[obj$input$data[,disp], "y2"][obj$input$subset],
              xlim = Xlim , ylim = Ylim ,
              main = obj$input$title,
              xlab = trunc_string(obj$input$xlab, obj$input$trunc_labels),
              ylab = trunc_string(obj$input$ylab, obj$input$trunc_labels),
+             cex.lab = obj$plot$par.settings$par.xlab.text$cex,
+             cex.main = obj$plot$par.settings$par.main.text$cex,
+             cex.axis = obj$plot$par.settings$axis.text$cex,
              pch = displayed[[disp]]$style, 
              col = displayed[[disp]][c("color","lightModeColor")][[obj$input$mode]],
              axes = FALSE)
         if(length(displayed) > 1) {
           for(disp in rev(disp_n)[-1]) {
-            points(x = obj$input$data[obj$input$data[,disp], "x2"], y = obj$input$data[obj$input$data[,disp], "y2"],
+            points(x = obj$input$data[obj$input$data[,disp], "x2"][obj$input$subset],
+                   y = obj$input$data[obj$input$data[,disp], "y2"][obj$input$subset],
                    pch = displayed[[disp]]$style, 
                    col = displayed[[disp]][c("color","lightModeColor")][[obj$input$mode]])
           }
         }
       } else {
-        groups = apply(as.data.frame(D[,disp_n]), 1, FUN=function(x) {
+        groups = apply(as.data.frame(D[obj$input$subset,disp_n]), 1, FUN=function(x) {
           tmp = which(x)[1]
           if(is.na(tmp)) return(NA)
           return(disp_n[tmp])
         })
         pch = sapply(groups, FUN = function(disp) displayed[[disp]]$style)
         col = sapply(groups, FUN = function(disp) displayed[[disp]][c("color","lightModeColor")][[obj$input$mode]])
-        plot(x = obj$input$data$x2, y = obj$input$data$y2,
-             xlim = Xlim, ylim = Ylim ,
+        plot(x = obj$input$data$x2[obj$input$subset],
+             y = obj$input$data$y2[obj$input$subset],
+             xlim = Xlim, ylim = Ylim,
              main = obj$input$title,
              xlab = trunc_string(obj$input$xlab, obj$input$trunc_labels),
              ylab = trunc_string(obj$input$ylab, obj$input$trunc_labels),
+             cex.lab = obj$plot$par.settings$par.xlab.text$cex,
+             cex.main = obj$plot$par.settings$par.main.text$cex,
+             cex.axis = obj$plot$par.settings$axis.text$cex,
              pch = pch, col = col,
              axes = FALSE)
       }
@@ -464,11 +506,14 @@ convert_to_baseplot = function(obj) {
   
   # axis
   if(pkg == "base") {
-    x_ticks = base_axis_constr(lim = Xlim, hyper = obj$input$trans_x, nint = n_ticks)
-    y_ticks = base_axis_constr(lim = Ylim, hyper = obj$input$trans_y, nint = n_ticks)
-    x_axis = axis(side = 1, at = x_ticks$at, labels = FALSE, cex.axis = 0.8)
-    text(x_axis, Ylim[1] - diff(Ylim) * 0.07, x_ticks$label, srt=45, xpd=TRUE, pos = 1, cex = 0.8)
-    axis(side = 2, at = y_ticks$at, labels = y_ticks$label, las = 2, cex.axis = 0.8) 
+    x_ticks = base_axis_constr(lim = Xlim, trans = obj$input$trans_x, nint = n_ticks)
+    y_ticks = base_axis_constr(lim = Ylim, trans = obj$input$trans_y, nint = n_ticks)
+    x_axis = axis(side = 1, at = x_ticks$at, labels = FALSE)
+    text(x = x_axis, y = Ylim[1] - diff(Ylim) * 0.07, labels = x_ticks$labels, xpd=TRUE, adj = c(1, 1),
+         cex = obj$plot$par.settings$axis.text$cex, cex.axis = obj$plot$par.settings$axis.text$cex, srt=45)
+    y_axis = axis(side = 2, at = y_ticks$at, labels = FALSE)
+    text(y = y_axis, x = Xlim[1] - diff(Xlim) * 0.07, labels = y_ticks$labels, xpd=TRUE, adj = c(1, 0.5),
+         cex = obj$plot$par.settings$axis.text$cex, cex.axis = obj$plot$par.settings$axis.text$cex, las=1)
     box()
   }
   
@@ -476,27 +521,27 @@ convert_to_baseplot = function(obj) {
   for(reg in obj$input$regions) {
     k = reg[c("color","lightcolor")][[obj$input$mode]]
     coords = reg[c("x","y")]
-    if(obj$input$trans_x!="P") {
-      coords$x = smoothLinLog(coords$x, hyper=obj$input$trans_x, base=10)
-      reg$cx = smoothLinLog(reg$cx, hyper=obj$input$trans_x, base=10)
-    }
+    trans_x = parseTrans(obj$input$trans_x)
+    coords$x = applyTrans(coords$x, trans_x)
+    reg$cx = applyTrans(reg$cx, trans_x)
     lab =  trunc_string(reg$label, obj$input$trunc_labels)
     if(reg$type=="line") {
+      if(reg$cy == 0) reg$cy = diff(Ylim)*0.6 # allow to show label when it is on the axe
+      if(coords$y[1] == 0) coords$y = rep(diff(Ylim)*.5, length.out=2) # allow to show line when on the axe
       switch(pkg,
              "lattice" = {
                foo = foo +
-                 layer(panel.text(x=reg$cx, y=reg$cy*diff(Ylim), col=k, labels=lab, pos=4)) +
+                 layer(panel.text(x=reg$cx, y=reg$cy*diff(Ylim), col=k, labels=lab, pos=4, cex=obj$plot$par.settings$add.text$cex)) +
                  layer(panel.lines(x=coords$x, y=coords$y*diff(Ylim),col=k))
              },
              "base" = {
-               text(x=reg$cx, y=reg$cy*diff(Ylim), col=k, labels=lab, pos=4)
+               text(x=reg$cx, y=reg$cy*diff(Ylim), col=k, labels=lab, pos=4, cex=obj$plot$par.settings$add.text$cex)
                polygon(x=coords$x, y=coords$y*diff(Ylim), col = k, border = k)
              })
     } else {
-      if(obj$input$trans_y!="P") {
-        coords$y = smoothLinLog(coords$y, hyper=obj$input$trans_y, base=10)
-        reg$cy = smoothLinLog(reg$cy, hyper=obj$input$trans_y, base=10)
-      }
+      trans_y = parseTrans(obj$input$trans_y)
+      coords$y = applyTrans(coords$y, trans_y)
+      reg$cy = applyTrans(reg$cy, trans_y)
       if(reg$type=="rect") {
         coords$x=c(coords$x[1],coords$x[1],coords$x[2],coords$x[2])
         coords$y=c(coords$y[1],coords$y[2],coords$y[2],coords$y[1])
@@ -507,11 +552,11 @@ convert_to_baseplot = function(obj) {
       switch(pkg,
              "lattice" = {
                foo = foo +
-                 layer(panel.text(x=reg$cx, y=reg$cy, col=k, labels=lab, pos=4)) +
+                 layer(panel.text(x=reg$cx, y=reg$cy, col=k, labels=lab, pos=4, cex=obj$plot$par.settings$add.text$cex)) +
                  layer(panel.polygon(x=coords$x, y=coords$y, border=k, col="transparent", lwd=1, lty=1))
              },
              "base" = {
-               text(x=reg$cx, y=reg$cy, col=k, labels=lab, pos=4) 
+               text(x=reg$cx, y=reg$cy, col=k, labels=lab, pos=4, cex=obj$plot$par.settings$add.text$cex) 
                polygon(x=coords$x, y=coords$y, border=k, col="transparent", lwd=1, lty=1)
              })
     }
@@ -523,12 +568,12 @@ convert_to_baseplot = function(obj) {
     legend("topleft", inset = 0.025, 
            lty = sapply(disp_n, FUN=function(p) c(1,2,3,4,6)[match(basepop[[obj$input$order[p]]]$linestyle,c("Solid","Dash","Dot","DashDot","DashDotDot"))]),
            col = sapply(displayed, FUN=function(p) p[c("color","lightModeColor")][[obj$input$mode]]),
-           legend = disp_n, cex = 0.5, bg = "#ADADAD99", pt.cex = 1, bty ="o", box.lty = 0)
+           legend = disp_n, cex = obj$plot$par.settings$add.text$cex * 0.5, bg = "#ADADAD99", pt.cex = 1, bty ="o", box.lty = 0)
   } else {
     legend("topleft", inset = 0.025, 
            pch = sapply(displayed, FUN=function(p) p$style),
            col = sapply(displayed, FUN=function(p) p[c("color","lightModeColor")][[obj$input$mode]]),
-           legend = disp_n, cex = 0.5, bg = "#ADADAD99", pt.cex = 1, bty ="o", box.lty = 0)
+           legend = disp_n, cex = obj$plot$par.settings$add.text$cex * 0.5, bg = "#ADADAD99", pt.cex = 1, bty ="o", box.lty = 0)
   }
 }
 
@@ -538,9 +583,10 @@ convert_to_baseplot = function(obj) {
 #' @param obj an object of class `IFC_data` extracted by ExtractFromDAF(extract_features = TRUE) or ExtractFromXIF(extract_features = TRUE).
 #' @param selection when provided, indices of desired graphs.\cr
 #' Note that indices are read from left to right, from top to bottom. 
+#' @param adjust_graph whether to try to adjust graph when possible. Default is TRUE.
 #' @param ... other arguments to be passed.
 #' @keywords internal
-adjustGraph = function(obj, selection, ...) {
+adjustGraph = function(obj, selection, adjust_graph = TRUE, ...) {
   dots = list(...)
   assert(obj, cla = "IFC_data")
   G = obj$graphs
@@ -559,19 +605,31 @@ adjustGraph = function(obj, selection, ...) {
         # check if y axis is present in obj
         if(g$type != "histogram") if(!(g$f2 %in% names(obj$features))) return(NULL)
         # check that at least one base pop will be plot
-        base_found = sapply(g$BasePop, FUN = function(p) p$name %in% names(obj$pops))
-        if(!any(base_found)) return(NULL)
+        tmp = sapply(g$BasePop, FUN = function(p) p$name %in% names(obj$pops))
+        if(!adjust_graph) if(!all(tmp)) return(NULL)
+        if(!any(tmp)) return(NULL)
         # remove BasePop not present in obj
-        g$BasePop = g$BasePop[base_found]
+        g$BasePop = g$BasePop[tmp]
         # remove GraphRegion not found in obj
-        if(length(g$GraphRegion)) g$GraphRegion = g$GraphRegion[sapply(g$GraphRegion, FUN = function(r) r$name %in% names(obj$regions))]
+        if(length(g$GraphRegion) !=0 && length(g$GraphRegion[[1]]) != 0) {
+          tmp = sapply(g$GraphRegion, FUN = function(r) r$name %in% names(obj$regions))
+          if(!adjust_graph) if(!all(tmp)) return(NULL)
+          g$GraphRegion = g$GraphRegion[tmp]
+        }
         # remove ShownPop not found in obj
-        if(length(g$ShownPop)) g$ShownPop = g$ShownPop[sapply(g$ShownPop, FUN = function(p) p$name %in% names(obj$pops))]
+        if(length(g$ShownPop) != 0 && length(g$ShownPop[[1]]) != 0) {
+          tmp = sapply(g$ShownPop, FUN = function(p) p$name %in% names(obj$pops))
+          if(!adjust_graph) if(!all(tmp)) return(NULL)
+          g$ShownPop = g$ShownPop[sapply(g$ShownPop, FUN = function(p) p$name %in% names(obj$pops))]
+        }
         # rebuild Graph, mainly to recompute order
-        return(do.call(what = buildGraph, args = g[!grepl("order", names(g))]))
-      } else {
-        return(g)
+        g = do.call(what = buildGraph, args = g[!grepl("order", names(g))])
+        if(inherits(x = g, what = "try-error")) return(NULL)
+        # try to draw the graph
+        drawable = plotGraph(obj = obj, graph = g, draw = FALSE, stats_print = FALSE)
+        if(inherits(x = drawable, what = "try-error")) return(NULL)
       }
+      return(g)
     })
     names(foo) = N
     bar = foo[sapply(foo, FUN = function(x) length(x) > 0)]
